"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path = require("path");
const Schema_1 = require("./Schema");
const NAMESPACE = "alexa";
const AVAILABLE_LOCALES = [
    "en-US",
    "en-GB",
    "en-CA",
    "en-AU",
    "en-IN",
    "de-DE",
    "ja-JP",
    "es-ES",
    "es-MX",
    "fr-FR",
    "fr-CA",
    "it-IT",
    "pt-BR"
];
class AlexaSchema extends Schema_1.Schema {
    constructor(voxaSheets, interactionOptions) {
        super(NAMESPACE, AVAILABLE_LOCALES, voxaSheets, interactionOptions);
        this.environment = "staging";
    }
    validate() { }
    build(locale, environment) {
        this.buildLanguageModel(locale, environment);
        this.buildPublishing(environment);
    }
    buildPublishing(environment) {
        const manifest = this.mergeManifest(environment);
        _.set(manifest, "manifestVersion", "1.0");
        this.fileContent.push({
            path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, `${_.kebabCase(environment)}-manifest.json`),
            content: { manifest }
        });
    }
    contentLanguageModel(locale, environment) {
        const invocation = _.find(this.invocations, { locale, environment });
        const invocationName = _.get(invocation, "name", "Skill with no name");
        const intents = this.getIntentsDefinition(locale, environment);
        const types = this.getSlotsByIntentsDefinition(locale, environment).map(rawSlot => {
            const { name, values } = rawSlot;
            const slot = { name, values: values.map(value => ({ name: value })) };
            return slot;
        });
        return { interactionModel: { languageModel: { invocationName, intents, types } } };
    }
    buildLanguageModel(locale, environment) {
        this.fileContent.push({
            path: path.join(this.interactionOptions.rootPath, this.interactionOptions.speechPath, this.NAMESPACE, locale, `${_.kebabCase(environment)}-interaction.json`),
            content: this.contentLanguageModel(locale, environment)
        });
        const canFulfillIntents = _.chain(this.intentsByPlatformAndEnvironments(locale, environment))
            .filter("canFulfillIntent")
            .map("name")
            .value();
        this.fileContent.push({
            path: path.join(this.interactionOptions.rootPath, this.interactionOptions.contentPath, `${_.kebabCase(environment)}-canfulfill-intents.json`),
            content: canFulfillIntents
        });
    }
}
exports.AlexaSchema = AlexaSchema;
//# sourceMappingURL=AlexaSchema.js.map