"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2018 Rain Agency <contact@rain.agency>
 * Author: Rain Agency <contact@rain.agency>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
/* tslint:disable:no-console no-submodule-imports */
const _Promise = require("bluebird");
const fsExtra = require("fs-extra");
const _ = require("lodash");
const path = require("path");
const AlexaSchema_1 = require("./AlexaSchema");
const Spreadsheet_1 = require("./connectors/Spreadsheet");
const DialogflowSchema_1 = require("./DialogflowSchema");
const Drive_1 = require("./Drive");
const fs = _Promise.promisifyAll(fsExtra);
exports.DEFAULT_INTERACTION_OPTIONS = {
    speechPath: "speech-assets",
    platforms: ["alexa"],
    contentPath: "content",
    viewsPath: "/",
    synonymPath: "synonyms",
    assets: [],
    assetsPath: "assets"
};
function defaultOptions(interactionOptions) {
    const rootPath = interactionOptions.rootPath || "";
    const speechPath = interactionOptions.speechPath || exports.DEFAULT_INTERACTION_OPTIONS.speechPath;
    const synonymPath = interactionOptions.synonymPath || exports.DEFAULT_INTERACTION_OPTIONS.synonymPath;
    const viewsPath = interactionOptions.viewsPath || exports.DEFAULT_INTERACTION_OPTIONS.viewsPath;
    const contentPath = interactionOptions.contentPath || exports.DEFAULT_INTERACTION_OPTIONS.contentPath;
    const assetsPath = interactionOptions.assetsPath || exports.DEFAULT_INTERACTION_OPTIONS.assetsPath;
    const assets = interactionOptions.assets || exports.DEFAULT_INTERACTION_OPTIONS.assets;
    const platforms = _.isString(interactionOptions.platforms)
        ? [interactionOptions.platforms]
        : interactionOptions.platforms || exports.DEFAULT_INTERACTION_OPTIONS.platforms;
    const spreadsheets = _.isString(interactionOptions.spreadsheets)
        ? [interactionOptions.spreadsheets]
        : interactionOptions.spreadsheets;
    if (_.isEmpty(spreadsheets)) {
        throw Error("Spreadsheet were not specified in the right format");
    }
    return {
        rootPath,
        spreadsheets,
        speechPath,
        synonymPath,
        viewsPath,
        assetsPath,
        contentPath,
        platforms,
        assets
    };
}
exports.buildInteraction = (interactionOptions, authKeys) => __awaiter(this, void 0, void 0, function* () {
    const definedInteractionOptions = defaultOptions(interactionOptions);
    console.time("all");
    console.time("timeframe");
    const sheets = yield Spreadsheet_1.transform(definedInteractionOptions, authKeys);
    console.timeEnd("timeframe");
    const platforms = definedInteractionOptions.platforms;
    const schemas = [];
    if (platforms.includes("alexa")) {
        const schema = new AlexaSchema_1.AlexaSchema(_.cloneDeep(sheets), definedInteractionOptions);
        schemas.push(schema);
        yield fs.remove(path.join(definedInteractionOptions.rootPath, definedInteractionOptions.speechPath, schema.NAMESPACE));
    }
    if (platforms.includes("dialogflow")) {
        const schema = new DialogflowSchema_1.DialogflowSchema(_.cloneDeep(sheets), definedInteractionOptions);
        schemas.push(schema);
        yield fs.remove(path.join(definedInteractionOptions.rootPath, definedInteractionOptions.speechPath, schema.NAMESPACE));
    }
    const fileContentsProcess = schemas
        .reduce((acc, schema, index) => {
        if (index === 0) {
            // We only want to execute this file once
            schema.buildDownloads();
            schema.buildViews();
            schema.buildViewsMapping();
            schema.buildSynonyms();
        }
        schema.invocations.map(invoc => {
            schema.build(invoc.locale, invoc.environment);
        });
        acc = acc.concat(schema.fileContent);
        return acc;
    }, [])
        .map(file => fs.outputFile(file.path, JSON.stringify(file.content, null, 2), { flag: "w" }));
    yield Promise.all(fileContentsProcess);
    yield Drive_1.downloadDirs(definedInteractionOptions.assets, path.join(definedInteractionOptions.rootPath, definedInteractionOptions.assetsPath), authKeys);
    console.timeEnd("all");
});
//# sourceMappingURL=InteractionBuilder.js.map