"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const _ = require("lodash");
const xlsx = require("node-xlsx");
const path = require("path");
const utils_1 = require("./utils");
function findLocalFiles(spreadsheet) {
    const fsStats = fs.lstatSync(spreadsheet);
    if (fsStats.isDirectory()) {
        return fs
            .readdirSync(spreadsheet)
            .filter(f => _.endsWith(f, ".xlsx") || _.endsWith(f, ".ods") || _.endsWith(f, ".fods"))
            .map(f => path.join(spreadsheet, f));
    }
    return [spreadsheet];
}
function readFileCreateWorkbook(f) {
    const workbook = xlsx.parse(f);
    const spreadsheetTitle = _.last(f.split("/"));
    const spreadsheetId = f;
    if (!_.isString(spreadsheetTitle)) {
        return undefined;
    }
    return workbook.map(book => ({
        spreadsheetId,
        spreadsheetTitle,
        sheetTitle: book.name,
        type: "none",
        data: processBookData(book.data).filter(row => row.length)
    }));
}
function refactorExcelData(sheet) {
    sheet.data = _.chain(sheet)
        .get("data")
        .map((next, index, arr) => {
        if (index === 0) {
            return next;
        }
        const head = arr[0];
        const extraColumns = head.length - next.length;
        if (extraColumns > 0) {
            next = _.concat(next, _.fill(Array(extraColumns), undefined));
        }
        return next;
    })
        .reduce(utils_1.rowFormatted, [])
        .drop()
        .value();
    return sheet;
}
function buildFromLocalExcel(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const vsheet = _.chain(options)
            .get("spreadsheets")
            .map(f => (f.indexOf("/") === 0 ? f : path.join(options.rootPath, f)))
            .filter(spreadsheet => fs.pathExistsSync(spreadsheet))
            .map(findLocalFiles)
            .flatten()
            .map(readFileCreateWorkbook)
            .flatten()
            .compact()
            .map(utils_1.findSheetType)
            .filter((sheet) => _.get(sheet, "type") !== "none" && !_.isEmpty(sheet))
            .map(refactorExcelData)
            .value();
        return vsheet;
    });
}
exports.buildFromLocalExcel = buildFromLocalExcel;
function processBookData(data) {
    // this is all because of a bug in node-xlsx where the first cell of the first tab get's a lot
    // of garbage appended to it's content
    const titleRow = data[0];
    const firstColumnTitle = titleRow[0];
    const split = firstColumnTitle.split("\n");
    if (split.length > 1) {
        titleRow[0] = split[split.length - 1];
    }
    return data;
}
//# sourceMappingURL=Excel.js.map