"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2018 Rain Agency <contact@rain.agency>
 * Author: Rain Agency <contact@rain.agency>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to
 * use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of
 * the Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
const _Promise = require("bluebird");
const google_auth_library_1 = require("google-auth-library");
const googleapis_1 = require("googleapis");
const _ = require("lodash");
const utils_1 = require("./utils");
const sheets = googleapis_1.google.sheets("v4");
const readSpreadsheet = _Promise.promisify(sheets.spreadsheets.get, { context: sheets });
const readSheetTab = _Promise.promisify(sheets.spreadsheets.values.get, { context: sheets });
function initVoxaSheet(spreadsheetsId, spreadsheetResp) {
    return _.chain(spreadsheetResp)
        .map((spreadsheet, index) => {
        const spreadsheetTitle = _.get(spreadsheet, "data.properties.title");
        const spreadsheetId = spreadsheetsId[index];
        const sheetNames = _.chain(spreadsheet)
            .get("data.sheets", [])
            .map("properties.title")
            .value();
        return sheetNames.map((sheetTitle) => {
            const voxaSheet = { spreadsheetId, spreadsheetTitle, sheetTitle, type: "none" };
            return voxaSheet;
        });
    })
        .flatten()
        .map(utils_1.findSheetType)
        .compact()
        .value();
}
function spreadsheetToVoxaSheet(client, spreadsheetsId, spreadsheetResp) {
    return __awaiter(this, void 0, void 0, function* () {
        spreadsheetResp = initVoxaSheet(spreadsheetsId, spreadsheetResp);
        let sheetPromises = spreadsheetResp.map((sheet) => readSheetTab({
            auth: client,
            spreadsheetId: sheet.spreadsheetId,
            range: `${sheet.sheetTitle}!A1:ZZZ`
        }));
        try {
            sheetPromises = yield _Promise.all(sheetPromises);
        }
        catch (e) {
            throw new Error(`Unable to get spreadsheet ${e}`);
        }
        return spreadsheetResp.map((sheet, index) => {
            const data = _.chain(sheetPromises[index])
                .get("data.values", [])
                .reduce(utils_1.rowFormatted, [])
                .drop()
                .value();
            // Apply processor
            sheet.data = data;
            return sheet;
        });
    });
}
function buildFromGoogleSheets(options, authKeys) {
    return __awaiter(this, void 0, void 0, function* () {
        const spreadsheetsId = _.chain(options)
            .get("spreadsheets")
            .map(getGoogleSpreadsheetId)
            .compact()
            .value();
        let spreadsheetResp = [];
        if (_.isEmpty(spreadsheetsId) || _.isEmpty(authKeys)) {
            return [];
        }
        const client = google_auth_library_1.auth.fromJSON(authKeys);
        client.scopes = ["https://www.googleapis.com/auth/spreadsheets.readonly"];
        try {
            spreadsheetResp = yield _Promise.all(spreadsheetsId.map((spreadsheetId) => readSpreadsheet({ auth: client, spreadsheetId })));
            // tslint:disable-next-line: no-empty
        }
        catch (e) {
            throw new Error(`Unable to read spreadsheets. Make sure user has access. ${e}`);
        }
        return spreadsheetToVoxaSheet(client, spreadsheetsId, spreadsheetResp);
    });
}
exports.buildFromGoogleSheets = buildFromGoogleSheets;
function getGoogleSpreadsheetId(sheet) {
    const matched = sheet.match(/docs\.google\.com\/spreadsheets\/d\/(.*)\/.*/);
    return sheet.includes("docs.google.com/spreadsheets") && matched && _.isString(matched[1])
        ? matched[1]
        : undefined;
}
//# sourceMappingURL=Google.js.map